import { BaseEvent } from '#application/base/baseEvent'
import DateManager from '#managers/dateManager'
import CharacterRepository from '#repositories/characterRepository'
import ChatService from '#services/chatService'

type TypePayload = {
  message: string
}

export default class SetTimeCommand extends BaseEvent {
  public listen(): void {
    this.socket.on('chat:message', this.handleEvent.bind(this))
  }

  private async handleEvent(data: TypePayload, callback: (response: boolean) => void): Promise<void> {
    try {
      if (!ChatService.isCommand(data.message, 'time')) {
        return
      }

      // Check if character exists
      const character = await CharacterRepository.getByUserAndId(this.socket.userId!, this.socket.characterId!)
      if (!character) {
        this.logger.error('chat:alert_command error', 'Character not found')
        return
      }

      // Check if the user is the GM
      if (character.role !== 'gm') {
        this.logger.info(`User ${character.id} tried to set time but is not a game master.`)
        return
      }

      // Get arguments
      const args = ChatService.getArgs('time', data.message)

      if (!args) {
        return
      }

      const time = args[0] // 24h time, e.g. 17:34

      if (!time) {
        return
      }

      await DateManager.setTime(time)
    } catch (error: any) {
      this.logger.error('command error', error.message)
      callback(false)
    }
  }
}
